/* 
 * Copyright (C) 2001-2005 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdinc.h"
#include "DCPlusPlus.h"

#include "SettingsManager.h"
#include "ResourceManager.h"

#include "SimpleXML.h"
#include "Util.h"
#include "File.h"
#include "version.h"
#include "CID.h"

const string SettingsManager::settingTags[] =
{
	// Strings
	"Connection", "Description", "DownloadDirectory", "EMail", "Nick", "Server",
	"Font", "MainFrameOrder", "MainFrameWidths", "HubFrameOrder", "HubFrameWidths", 
	"LanguageFile", "SearchFrameOrder", "SearchFrameWidths", "FavoritesFrameOrder", "FavoritesFrameWidths", 
	"HublistServers", "QueueFrameOrder", "QueueFrameWidths", "PublicHubsFrameOrder", "PublicHubsFrameWidths", 
	"UsersFrameOrder", "UsersFrameWidths", "HttpProxy", "LogDirectory", "NotepadText", "LogFormatPostDownload", 
	"LogFormatPostUpload", "LogFormatMainChat", "LogFormatPrivateChat", "FinishedOrder", "FinishedWidths",	 
	"TempDownloadDirectory", "BindAddress", "SocksServer", "SocksUser", "SocksPassword", "ConfigVersion", 
	"DefaultAwayMessage", "ADLSearchFrameOrder", "ADLSearchFrameWidths", "CID", "SpyFrameWidths", "SpyFrameOrder", 
	"FinishedULWidths", "FinishedULOrder", "TimeStampsFormat", "LogFileMainChat", "LogFilePrivateChat", 
	"LogFileStatus", "LogFileUpload", "LogFileDownload", "LogFileSystem", "LogFormatSystem", "LogFormatStatus", 
	"DirectoryListingFrameOrder", "DirectoryListingFrameWidths", 
	"BeepFile", "WinampFormat", "Toolbar", "EmoticonFile", 
	"KickMsgRecent01", "KickMsgRecent02", "KickMsgRecent03", "KickMsgRecent04", "KickMsgRecent05", 
	"KickMsgRecent06", "KickMsgRecent07", "KickMsgRecent08", "KickMsgRecent09", "KickMsgRecent10", 
	"KickMsgRecent11", "KickMsgRecent12", "KickMsgRecent13", "KickMsgRecent14", "KickMsgRecent15", 
	"KickMsgRecent16", "KickMsgRecent17", "KickMsgRecent18", "KickMsgRecent19", "KickMsgRecent20", 
	"SearchMsgRecent01", "SearchMsgRecent02", "SearchMsgRecent03", "SearchMsgRecent04", "SearchMsgRecent05", 
	"SearchMsgRecent06", "SearchMsgRecent07", "SearchMsgRecent08", "SearchMsgRecent09", "SearchMsgRecent10", 
	"SearchMsgRecent11", "SearchMsgRecent12", "SearchMsgRecent13", "SearchMsgRecent14", "SearchMsgRecent15", 
	"SearchMsgRecent16", "SearchMsgRecent17", "SearchMsgRecent18", "SearchMsgRecent19", "SearchMsgRecent20", 
	"RecentHubsFrameOrder", "RecentHubsFrameWidths", "UploadQueueFrameOrder", "UploadQueueFrameWidths", 
	"MainFrameVisible", "SearchFrameVisible", "QueueFrameVisible", "HubFrameVisible", "UploadQueueFrameVisible", 
	"SENTRY", 
	// Ints
	"ConnectionType", "InPort", "Slots", "Rollback", "AutoFollow", "ClearSearch", 
	"BackgroundColor", "TextColor", "UseOemMonoFont", "ShareHidden", "FilterMessages", "MinimizeToTray", 
	"AutoSearch", "TimeStamps", "ConfirmExit", "IgnoreOffline", "PopupOffline", 
	"ListDuplicates", "BufferSize", "DownloadSlots", "MaxDownloadSpeed", "LogMainChat", "LogPrivateChat", 
	"LogDownloads", "LogUploads", "StatusInChat", "ShowJoins", "PrivateMessageBeep", "PrivateMessageBeepOpen", 
	"UseSystemIcons", "PopupPMs", "MinUploadSpeed", "GetUserInfo", "UrlHandler", "MainWindowState", 
	"MainWindowSizeX", "MainWindowSizeY", "MainWindowPosX", "MainWindowPosY", "AutoAway", 
	"SmallSendBuffer", "SocksPort", "SocksResolve", "KeepLists", "AutoKick", "QueueFrameShowTree", 
	"CompressTransfers", "ShowProgressBars", "SFVCheck", "MaxTabRows", "AutoUpdateList", 
	"MaxCompression", "FinishedDirty", "QueueDirty", "AntiFrag", "MDIMaxmimized", "UDPPort", 
	"SkipZeroByte", "AdlsBreakOnFirst", "TabCompletion", 
	"HubUserCommands", "AutoSearchAutoMatch", "UploadBarColor", "DownloadBarColor", "LogSystem", 
	"LogFilelistTransfers", "SendUnknownCommands", "MaxHashSpeed", "UseCTRLForLineHistory", 
	"GetUserCountry", "FavShowJoins", "LogStatusMessages", "ShowStatusbar", "OpenNewWindow", 
	"ShowToolbar", "ShowTransferview", "PopunderPm", "PopunderFilelist", "TabDirty", "MagnetAsk", 
	"MagnetAction", "MagnetRegister", "AddFinishedInstantly", "UseUPnP", "DontDLAlreadyShared", 
	"SearchOnlyTTH", "AdvancedResume", "ToggleActiveWindow", "HighestPrioSize", "HighPrioSize", 
	"NormalPrioSize", "LowPrioSize", "LowestPrio", 
	"OpenPublic", "OpenFavoriteHubs", "OpenFavoriteUsers", "OpenQueue", "OpenFinishedDownloads",
	"OpenFinishedUploads", "OpenSearchSpy", "OpenNotepad",
	"ExtraInfo", "GetUpdateInfo", "SearchPassiveAlways", "SmallFileSize", "ShutdownInterval", 
	"CzertHiddenSettingA", "FilterSearchResults", "CzertHiddenSettingB", "ExtraSlots", 
	"TextGeneralBackColor", "TextGeneralForeColor", "TextGeneralBold", "TextGeneralItalic", 
	"TextMyOwnBackColor", "TextMyOwnForeColor", "TextMyOwnBold", "TextMyOwnItalic", 
	"TextPrivateBackColor", "TextPrivateForeColor", "TextPrivateBold", "TextPrivateItalic", 
	"TextSystemBackColor", "TextSystemForeColor", "TextSystemBold", "TextSystemItalic", 
	"TextServerBackColor", "TextServerForeColor", "TextServerBold", "TextServerItalic", 
	"TextTimestampBackColor", "TextTimestampForeColor", "TextTimestampBold", "TextTimestampItalic", 
	"TextMyNickBackColor", "TextMyNickForeColor", "TextMyNickBold", "TextMyNickItalic", 
	"TextFavBackColor", "TextFavForeColor", "TextFavBold", "TextFavItalic", 
	"TextURLBackColor", "TextURLForeColor", "TextURLBold", "TextURLItalic", 
	"BoldAuthorsMess", "UploadLimitNormal", "ThrottleEnable", "DownloadLimitNormal", 
	"UploadLimitTime", "DownloadLimitTime", "TimeThrottle", "TimeLimitStart", "TimeLimitEnd", 
	"ProgressTextDown", "ProgressTextUp", "ShowInfoTips", "ExtraDownloadSlots", 
	"MinimizeOnStratup", "ConfirmDelete", "DefaultSearchFreeSlots", 
	"ExtensionDownTo", "ErrorColor", "ExpandQueue", "TransferSplitSize", "ReportFoundAlternates", 
	"ShowIP", "IDownSpeed", "HDownSpeed", "DownTime", "DisconnectingEnable", "MinFileSize", "RemoveSlowUser", 
    "ShowShareCheat", "PMLogLines", "UseEmoticons", "MaxEmoticons", 
	"TextOPBackColor", "TextOPForeColor", "TextOPBold", "TextOPItalic", "ClientCommands", 
	"ServerCommands", "UserDownload", "ReportDownload", "ReportUpload", "UploadQueueFrameShowTree", 
	"GarbageIn", "GarbageOut", "TTHCheck", "ConfirmHubExit", "AwayEnabled", "AlternateUploadSlots", 
	"TabsOnTop", "MuteSound", "PausedColor", "UserListDoubleClick", "TransferListDoubleClick", 
	"ChatDoubleClick", "StripTopic", "UserlistColoring", "FavoriteColor", "ExtraSlotColor", "IgnoredColor", 
	"FireballColor", "ServerColor", "ActiveColor", "PasiveColor", "OpColor", "NormalColor", "ConfirmHubReconnect", 
	"LogPartialUploads", "OpenNetworkStatistic", "OpenRecentHubs", "OpenUploadQueue", "GroupSearchResults",
	"SearchExpand", "FileListDownloadslots", "MaxFileList2Match", "AutoSearchOnAway", 
	"SENTRY",
	// Int64
	"TotalUpload", "TotalDownload", "TotalUptime", 
	"SENTRY"
};

const string SettingsManager::connectionSpeeds[] = { "Modem", "ISDN", 
"Satellite", "Wireless", "Cable", "DSL", "LAN(T1)", "LAN(T3)" };

SettingsManager::SettingsManager()
{
	for(int i=0; i<SETTINGS_LAST; i++)
		isSet[i] = false;

	for(int j=0; j<INT_LAST-INT_FIRST; j++) {
		intDefaults[j] = 0;
		intSettings[j] = 0;
	}
	for(int k=0; k<INT64_LAST-INT64_FIRST; k++) {
		int64Defaults[k] = 0;
		int64Settings[k] = 0;
	}
	
	setDefault(DOWNLOAD_DIRECTORY, Util::getAppPath() + "Downloads" PATH_SEPARATOR_STR);
	setDefault(SLOTS, 1);
	setDefault(CONNECTION_TYPE, 1);
	setDefault(IN_PORT, Util::rand(1025, 32000));
	setDefault(UDP_PORT, Util::rand(1025, 32000));
	setDefault(ROLLBACK, 4096);
	setDefault(AUTO_FOLLOW, true);
	setDefault(CLEAR_SEARCH, false);
	setDefault(SHARE_HIDDEN, false);
	setDefault(FILTER_MESSAGES, true);
	setDefault(MINIMIZE_TRAY, true);
	setDefault(AUTO_SEARCH, false);
	setDefault(TIME_STAMPS, true);
	setDefault(CONFIRM_EXIT, false);
	setDefault(IGNORE_OFFLINE, false);
	setDefault(POPUP_OFFLINE, false);
	setDefault(LIST_DUPES, true);
	setDefault(BUFFER_SIZE, 64);
	setDefault(HUBLIST_SERVERS, "http://www.hublist.org/PublicHubList.xml.bz2;http://dc.selwerd.nl/hublist.xml.bz2");
	setDefault(DOWNLOAD_SLOTS, 3);
	setDefault(MAX_DOWNLOAD_SPEED, 0);
	setDefault(LOG_DIRECTORY, Util::getAppPath() + "Logs" PATH_SEPARATOR_STR);
	setDefault(LOG_UPLOADS, false);
	setDefault(LOG_DOWNLOADS, false);
	setDefault(LOG_PRIVATE_CHAT, false);
	setDefault(LOG_MAIN_CHAT, false);
	setDefault(STATUS_IN_CHAT, true);
	setDefault(SHOW_JOINS, false);
	setDefault(CONNECTION, connectionSpeeds[0]);
	setDefault(PRIVATE_MESSAGE_BEEP, false);
	setDefault(PRIVATE_MESSAGE_BEEP_OPEN, false);
	setDefault(USE_SYSTEM_ICONS, true);
	setDefault(USE_OEM_MONOFONT, false);
	setDefault(POPUP_PMS, true);
	setDefault(MIN_UPLOAD_SPEED, 0);
	setDefault(LOG_FORMAT_POST_DOWNLOAD, "%Y-%m-%d %H:%M: %[target]" + STRING(DOWNLOADED_FROM) + "%[user], %[size] (%[chunksize]), %[speed], %[time], %[tth]");
	setDefault(LOG_FORMAT_POST_UPLOAD, "%Y-%m-%d %H:%M: %[source]" + STRING(UPLOADED_TO) + "%[user], %[size] (%[chunksize]), %[speed], %[time]");
	setDefault(LOG_FORMAT_MAIN_CHAT, "[%Y-%m-%d %H:%M] %[message]");
	setDefault(LOG_FORMAT_PRIVATE_CHAT, "[%Y-%m-%d %H:%M] %[message]");
	setDefault(LOG_FORMAT_STATUS, "[%Y-%m-%d %H:%M] %[message]");
	setDefault(LOG_FORMAT_SYSTEM, "[%Y-%m-%d %H:%M] %[message]");
	setDefault(LOG_FILE_MAIN_CHAT, "%[hubaddr].log");
	setDefault(LOG_FILE_STATUS, "%[hubaddr]_status.log");
	setDefault(LOG_FILE_PRIVATE_CHAT, "%[user].log");
	setDefault(LOG_FILE_UPLOAD, "Uploads.log");
	setDefault(LOG_FILE_DOWNLOAD, "Downloads.log");
	setDefault(LOG_FILE_SYSTEM, "system.log");
	setDefault(GET_USER_INFO, true);
	setDefault(URL_HANDLER, false);
	setDefault(AUTO_AWAY, false);
	setDefault(SMALL_SEND_BUFFER, false);
	setDefault(BIND_ADDRESS, "0.0.0.0");
	setDefault(SOCKS_PORT, 1080);
	setDefault(SOCKS_RESOLVE, 1);
	setDefault(CONFIG_VERSION, "0.181");		// 0.181 is the last version missing configversion
	setDefault(KEEP_LISTS, false);
	setDefault(AUTO_KICK, false);
	setDefault(QUEUEFRAME_SHOW_TREE, true);
	setDefault(COMPRESS_TRANSFERS, true);
	setDefault(SHOW_PROGRESS_BARS, true);
	setDefault(SFV_CHECK, false);
	setDefault(DEFAULT_AWAY_MESSAGE, "I'm away. I might answer later if you're lucky.");
	setDefault(TIME_STAMPS_FORMAT, "%H:%M:%S");
	setDefault(MAX_TAB_ROWS, 2);
	setDefault(AUTO_UPDATE_LIST, true);
	setDefault(MAX_COMPRESSION, 6);
	setDefault(FINISHED_DIRTY, true);
	setDefault(QUEUE_DIRTY, true);
	setDefault(TAB_DIRTY, true);
	setDefault(ANTI_FRAG, false);
	setDefault(SKIP_ZERO_BYTE, true);
	setDefault(ADLS_BREAK_ON_FIRST, false);
	setDefault(TAB_COMPLETION, true);
	setDefault(HUB_USER_COMMANDS, true);
	setDefault(AUTO_SEARCH_AUTO_MATCH, true);
	setDefault(LOG_FILELIST_TRANSFERS, true);
	setDefault(LOG_SYSTEM, false);
	setDefault(SEND_UNKNOWN_COMMANDS, true);
	setDefault(MAX_HASH_SPEED, 0);
	setDefault(GET_USER_COUNTRY, true);
	setDefault(FAV_SHOW_JOINS, false);
	setDefault(LOG_STATUS_MESSAGES, false);
	setDefault(SHOW_TRANSFERVIEW, true);
	setDefault(SHOW_STATUSBAR, true);
	setDefault(SHOW_TOOLBAR, true);
	setDefault(POPUNDER_PM, false);
	setDefault(POPUNDER_FILELIST, false);
	setDefault(MAGNET_REGISTER, true);
	setDefault(MAGNET_ASK, true);
	setDefault(MAGNET_ACTION, MAGNET_AUTO_SEARCH);
	setDefault(ADD_FINISHED_INSTANTLY, false);
	setDefault(SETTINGS_USE_UPNP, false);
	setDefault(DONT_DL_ALREADY_SHARED, false);
	setDefault(SETTINGS_USE_CTRL_FOR_LINE_HISTORY, true);
	setDefault(SETTINGS_OPEN_NEW_WINDOW, false);
	setDefault(SEARCH_ONLY_TTH, false);
	setDefault(ADVANCED_RESUME, true);
	setDefault(TOGGLE_ACTIVE_WINDOW, true);
	setDefault(PRIO_HIGHEST_SIZE, 64);
	setDefault(PRIO_HIGH_SIZE, 0);
	setDefault(PRIO_NORMAL_SIZE, 0);
	setDefault(PRIO_LOW_SIZE, 0);
	setDefault(PRIO_LOWEST, 0);
	setDefault(OPEN_PUBLIC, false);
	setDefault(OPEN_FAVORITE_HUBS, false);
	setDefault(OPEN_FAVORITE_USERS, false);
	setDefault(OPEN_QUEUE, false);
	setDefault(OPEN_FINISHED_DOWNLOADS, false);
	setDefault(OPEN_FINISHED_UPLOADS, false);
	setDefault(OPEN_SEARCH_SPY, false);
	setDefault(OPEN_NOTEPAD, false);
	
	setDefault(EXTRA_SLOTS, 3);
	setDefault(SMALL_FILE_SIZE, 256);
	setDefault(SEND_EXTENDED_INFO, true);
	setDefault(GET_UPDATE_INFO, true);
	setDefault(SEARCH_PASSIVE, false);
	setDefault(FILTER_SEARCH, true);
	setDefault(MAX_UPLOAD_SPEED_LIMIT_NORMAL, 0);
	setDefault(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL, 0);
	setDefault(MAX_UPLOAD_SPEED_LIMIT_TIME, 0);
	setDefault(MAX_DOWNLOAD_SPEED_LIMIT_TIME, 0);
	setDefault(TIME_DEPENDENT_THROTTLE, false);
	setDefault(BANDWIDTH_LIMIT_START, 0);
	setDefault(BANDWIDTH_LIMIT_END, 0);
	setDefault(THROTTLE_ENABLE, false);
	setDefault(EXTRA_DOWNLOAD_SLOTS, 3);

	setDefault(BACKGROUND_COLOR, RGB(0,0,96));
	setDefault(TEXT_COLOR, RGB(255,255,255));

	setDefault(TEXT_GENERAL_BACK_COLOR, RGB(0,0,96));
	setDefault(TEXT_GENERAL_FORE_COLOR, RGB(255,255,255));
	setDefault(TEXT_GENERAL_BOLD, false);
	setDefault(TEXT_GENERAL_ITALIC, false);

	setDefault(TEXT_MYOWN_BACK_COLOR, RGB(96,0,0));
	setDefault(TEXT_MYOWN_FORE_COLOR, RGB(255,255,0));
	setDefault(TEXT_MYOWN_BOLD, true);
	setDefault(TEXT_MYOWN_ITALIC, false);

	setDefault(TEXT_PRIVATE_BACK_COLOR, RGB(0,96,0));
	setDefault(TEXT_PRIVATE_FORE_COLOR, RGB(255,255,255));
	setDefault(TEXT_PRIVATE_BOLD, true);
	setDefault(TEXT_PRIVATE_ITALIC, false);

	setDefault(TEXT_SYSTEM_BACK_COLOR, RGB(0,0,0));
	setDefault(TEXT_SYSTEM_FORE_COLOR, RGB(192,192,192));
	setDefault(TEXT_SYSTEM_BOLD, true);
	setDefault(TEXT_SYSTEM_ITALIC, true);

	setDefault(TEXT_SERVER_BACK_COLOR, RGB(0,0,0));
	setDefault(TEXT_SERVER_FORE_COLOR, RGB(128,255,128));
	setDefault(TEXT_SERVER_BOLD, true);
	setDefault(TEXT_SERVER_ITALIC, false);

	setDefault(TEXT_TIMESTAMP_BACK_COLOR, RGB(0,0,0));
	setDefault(TEXT_TIMESTAMP_FORE_COLOR, RGB(255,255,0));
	setDefault(TEXT_TIMESTAMP_BOLD, false);
	setDefault(TEXT_TIMESTAMP_ITALIC, false);

	setDefault(TEXT_MYNICK_BACK_COLOR, RGB(0,0,96));
	setDefault(TEXT_MYNICK_FORE_COLOR, RGB(255,255,0));
	setDefault(TEXT_MYNICK_BOLD, true);
	setDefault(TEXT_MYNICK_ITALIC, false);

	setDefault(TEXT_FAV_BACK_COLOR, RGB(0,0,0));
	setDefault(TEXT_FAV_FORE_COLOR, RGB(255,128,128));
	setDefault(TEXT_FAV_BOLD, true);
	setDefault(TEXT_FAV_ITALIC, true);

	setDefault(TEXT_URL_BACK_COLOR, RGB(192,192,192));
	setDefault(TEXT_URL_FORE_COLOR, RGB(0,0,255));
	setDefault(TEXT_URL_BOLD, false);
	setDefault(TEXT_URL_ITALIC, false);

	setDefault(BOLD_AUTHOR_MESS, true);
	setDefault(KICK_MSG_RECENT_01, "");
	setDefault(KICK_MSG_RECENT_02, "");
	setDefault(KICK_MSG_RECENT_03, "");
	setDefault(KICK_MSG_RECENT_04, "");
	setDefault(KICK_MSG_RECENT_05, "");
	setDefault(KICK_MSG_RECENT_06, "");
	setDefault(KICK_MSG_RECENT_07, "");
	setDefault(KICK_MSG_RECENT_08, "");
	setDefault(KICK_MSG_RECENT_09, "");
	setDefault(KICK_MSG_RECENT_10, "");
	setDefault(KICK_MSG_RECENT_11, "");
	setDefault(KICK_MSG_RECENT_12, "");
	setDefault(KICK_MSG_RECENT_13, "");
	setDefault(KICK_MSG_RECENT_14, "");
	setDefault(KICK_MSG_RECENT_15, "");
	setDefault(KICK_MSG_RECENT_16, "");
	setDefault(KICK_MSG_RECENT_17, "");
	setDefault(KICK_MSG_RECENT_18, "");
	setDefault(KICK_MSG_RECENT_19, "");
	setDefault(KICK_MSG_RECENT_20, "");
	setDefault(WINAMP_FORMAT, "winamp(%[version]) %[state](%[title]) stats(%[percent] of %[length] %[bar])");
	setDefault(PROGRESS_TEXT_COLOR_DOWN, RGB(255, 255, 255));
	setDefault(PROGRESS_TEXT_COLOR_UP, RGB(255, 255, 255));
	setDefault(SHOW_INFOTIPS, true);
	setDefault(MINIMIZE_ON_STARTUP, false);
	setDefault(CONFIRM_DELETE, true);
	setDefault(FREE_SLOTS_DEFAULT, false);
	setDefault(USE_EXTENSION_DOWNTO, true);
	setDefault(ERROR_COLOR, RGB(255, 0, 0));
	setDefault(EXPAND_QUEUE, true);
	setDefault(TRANSFER_SPLIT_SIZE, 8000);
	setDefault(SHOW_IP, false);
	setDefault(I_DOWN_SPEED, 0);
	setDefault(H_DOWN_SPEED, 0);
	setDefault(DOWN_TIME, 1);
	setDefault(DISCONNECTING_ENABLE, false);
	setDefault(MIN_FILE_SIZE, 0);
	setDefault(REMOVE_SLOW_USER, false);
	setDefault(SHUTDOWN_TIMEOUT, 300);
	setDefault(SHOW_SHARE_CHEAT, true);
	setDefault(TOOLBAR, "0,-1,19,-1,1,-1,2,3,4,20,-1,5,-1,6,7,-1,8,9,10,11,-1,12,13,14,-1,15,-1,16,-1,17,-1,18");
	setDefault(PM_LOG_LINES, 10);
	setDefault(USE_EMOTICONS, true);
	setDefault(MAX_EMOTICONS, 256);
	setDefault(TEXT_OP_BACK_COLOR, RGB(0,0,0));
	setDefault(TEXT_OP_FORE_COLOR, RGB(255,255,255));
	setDefault(TEXT_OP_BOLD, true);
	setDefault(TEXT_OP_ITALIC, false);
	setDefault(CLIENT_COMMANDS, true);
	setDefault(SERVER_COMMANDS, true);
	setDefault(USER_COMMANDS, true);
	setDefault(REPORT_DOWNLOAD, true);
	setDefault(REPORT_UPLOAD, true);
	setDefault(REPORT_ALTERNATES, true);
	setDefault(GARBAGE_COMMAND_INCOMING, false);
	setDefault(GARBAGE_COMMAND_OUTGOING, false);
	setDefault(TTH_CHECK, true);
	setDefault(UPLOADQUEUEFRAME_SHOW_TREE, true);
	setDefault(CONFIRM_HUBEXIT, false);
	setDefault(BEEPFILE, "");
	setDefault(EMOTICONFILE, "");
	setDefault(SEARCH_MSG_RECENT_01, "");
	setDefault(SEARCH_MSG_RECENT_02, "");
	setDefault(SEARCH_MSG_RECENT_03, "");
	setDefault(SEARCH_MSG_RECENT_04, "");
	setDefault(SEARCH_MSG_RECENT_05, "");
	setDefault(SEARCH_MSG_RECENT_06, "");
	setDefault(SEARCH_MSG_RECENT_07, "");
	setDefault(SEARCH_MSG_RECENT_08, "");
	setDefault(SEARCH_MSG_RECENT_09, "");
	setDefault(SEARCH_MSG_RECENT_10, "");
	setDefault(SEARCH_MSG_RECENT_11, "");
	setDefault(SEARCH_MSG_RECENT_12, "");
	setDefault(SEARCH_MSG_RECENT_13, "");
	setDefault(SEARCH_MSG_RECENT_14, "");
	setDefault(SEARCH_MSG_RECENT_15, "");
	setDefault(SEARCH_MSG_RECENT_16, "");
	setDefault(SEARCH_MSG_RECENT_17, "");
	setDefault(SEARCH_MSG_RECENT_18, "");
	setDefault(SEARCH_MSG_RECENT_19, "");
	setDefault(SEARCH_MSG_RECENT_20, "");
	setDefault(AWAY_ENABLED, false);
	setDefault(ALTERNATE_UPLOAD_SLOTS, 0);
	setDefault(TABS_ON_TOP, false);
	setDefault(MUTE_SOUND, false);
	setDefault(PAUSED_COLOR, RGB(0, 255, 0));
	setDefault(USERLIST_DBLCLICK, 0);
	setDefault(TRANSFERLIST_DBLCLICK, 0);
	setDefault(CHAT_DBLCLICK, 0);
	setDefault(STRIP_TOPIC, false);
	setDefault(USERLIST_COLORING, true);
	setDefault(NORMAL_COLOR, RGB(255,255,255));
	setDefault(FAVORITE_COLOR, RGB(82,220,206));
	setDefault(EXTRA_SLOT_COLOR, RGB(0,255,0));
	setDefault(IGNORED_COLOR, RGB(0,0,0));
	setDefault(FIREBALL_COLOR, RGB(255,100,0));
 	setDefault(SERVER_COLOR, RGB(128,128,255));
	setDefault(ACTIVE_COLOR, RGB(255,255,255));
	setDefault(PASIVE_COLOR, RGB(255,255,255));
	setDefault(OP_COLOR, RGB(255,255,0));
	setDefault(CONFIRM_HUB_RECONNECT, false);
	setDefault(LOG_PARTIAL_UPLOADS, false);
	setDefault(HUBFRAME_VISIBLE, "1,1,0,1,0,1,1,0,0,0,0,0,0,0");
	setDefault(OPEN_NETWORK_STATISTIC, false);
	setDefault(OPEN_RECENT_HUBS, false);
	setDefault(OPEN_UPLOAD_QUEUE, false);
	setDefault(GROUP_SEARCH_RESULTS, true);
	setDefault(SEARCH_EXPAND, false);
	setDefault(FILELIST_DOWNLOAD_SLOTS, 3);
	setDefault(MAX_FILELIST2MATCH, 0);
	setDefault(AUTO_SEARCH_ON_AWAY, false);
#ifdef _WIN32
	setDefault(MAIN_WINDOW_STATE, SW_SHOWNORMAL);
	setDefault(MAIN_WINDOW_SIZE_X, CW_USEDEFAULT);
	setDefault(MAIN_WINDOW_SIZE_Y, CW_USEDEFAULT);
	setDefault(MAIN_WINDOW_POS_X, CW_USEDEFAULT);
	setDefault(MAIN_WINDOW_POS_Y, CW_USEDEFAULT);
	setDefault(MDI_MAXIMIZED, true);
	setDefault(UPLOAD_BAR_COLOR, RGB(160, 0, 0));
	setDefault(DOWNLOAD_BAR_COLOR, RGB(0, 160, 0));
#endif
}

void SettingsManager::load(string const& aFileName)
{
	try {
		SimpleXML xml;
		
		xml.fromXML(File(aFileName, File::READ, File::OPEN).read());
		
		xml.resetCurrentChild();
		
		xml.stepIn();
		
		if(xml.findChild("Settings"))
		{
			xml.stepIn();
			
			int i;
			
			for(i=STR_FIRST; i<STR_LAST; i++)
			{
				const string& attr = settingTags[i];
				dcassert(attr.find("SENTRY") == string::npos);
				
				if(xml.findChild(attr))
					set(StrSetting(i), xml.getChildData());
				xml.resetCurrentChild();
			}
			for(i=INT_FIRST; i<INT_LAST; i++)
			{
				const string& attr = settingTags[i];
				dcassert(attr.find("SENTRY") == string::npos);
				
				if(xml.findChild(attr))
					set(IntSetting(i), Util::toInt(xml.getChildData()));
				xml.resetCurrentChild();
			}
			for(i=INT64_FIRST; i<INT64_LAST; i++)
			{
				const string& attr = settingTags[i];
				dcassert(attr.find("SENTRY") == string::npos);
				
				if(xml.findChild(attr))
					set(Int64Setting(i), Util::toInt64(xml.getChildData()));
				xml.resetCurrentChild();
			}
			
			xml.stepOut();
		}

		double v = Util::toDouble(SETTING(CONFIG_VERSION));
		// if(v < 0.x) { // Fix old settings here }
		if(v < 0.666 && isSet[IN_PORT]) {
			set(UDP_PORT, SETTING(IN_PORT));
		}

		if(CID(SETTING(CLIENT_ID)).isZero())
			set(CLIENT_ID, CID::generate().toBase32());

		if(SETTING(PRIO_HIGHEST_SIZE) < 16)
			set(PRIO_HIGHEST_SIZE, 16);
		if(SETTING(PRIO_HIGH_SIZE) < 0)
			set(PRIO_HIGH_SIZE, 0);
		if(SETTING(PRIO_NORMAL_SIZE) < 0)
			set(PRIO_NORMAL_SIZE, 0);
		if(SETTING(PRIO_LOW_SIZE) < 0)
			set(PRIO_LOW_SIZE, 0);
		if(SETTING(PRIO_LOWEST) < 0)
			set(PRIO_LOWEST, 0);
		if(SETTING(PRIO_LOWEST) > 1)
			set(PRIO_LOWEST, 1);
		setDefault(UDP_PORT, SETTING(IN_PORT));

		fire(SettingsManagerListener::Load(), &xml);

		xml.stepOut();

	} catch(const Exception&) {
		if(CID(SETTING(CLIENT_ID)).isZero())
			set(CLIENT_ID, CID::generate().toBase32());
	}
}

void SettingsManager::save(string const& aFileName) {

	SimpleXML xml;
	xml.addTag("DCPlusPlus");
	xml.stepIn();
	xml.addTag("Settings");
	xml.stepIn();

	int i;
	string type("type"), curType("string");
	
	for(i=STR_FIRST; i<STR_LAST; i++)
	{
		if(i == CONFIG_VERSION) {
			xml.addTag(settingTags[i], VERSIONSTRING);
			xml.addChildAttrib(type, curType);
		} else if(isSet[i]) {
			xml.addTag(settingTags[i], get(StrSetting(i), false));
			xml.addChildAttrib(type, curType);
		}
	}

	curType = "int";
	for(i=INT_FIRST; i<INT_LAST; i++)
	{
		if(isSet[i]) {
			xml.addTag(settingTags[i], get(IntSetting(i), false));
			xml.addChildAttrib(type, curType);
		}
	}
	curType = "int64";
	for(i=INT64_FIRST; i<INT64_LAST; i++)
	{
		if(isSet[i])
		{
			xml.addTag(settingTags[i], get(Int64Setting(i), false));
			xml.addChildAttrib(type, curType);
		}
	}
	xml.stepOut();
	
	fire(SettingsManagerListener::Save(), &xml);

	try {
		File out(aFileName + ".tmp", File::WRITE, File::CREATE | File::TRUNCATE);
		BufferedOutputStream<false> f(&out);
		f.write(SimpleXML::utf8Header);
		xml.toXML(&f);
		f.flush();
		out.close();
		File::deleteFile(aFileName);
		File::renameFile(aFileName + ".tmp", aFileName);
	} catch(const FileException&) {
		// ...
	}
}

/* Some parts Copyright (C) 2002-2005 Petr Kozelka, PPK at seznam dot cz */
