/* 
 * Copyright (C) 2002-2005 Petr Kozelka, PPK at seznam dot cz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "PropPageTextStyles.h"
#include "../client/SettingsManager.h"
#include "WinUtil.h"

PropPage::TextItem PropPageTextStyles::texts[] = {
	{ IDC_CZDC_AVAILABLE_STYLES, ResourceManager::SETCZDC_STYLES },
	{ IDC_BACK_COLOR, ResourceManager::SETCZDC_BACK_COLOR },
	{ IDC_TEXT_COLOR, ResourceManager::SETCZDC_TEXT_COLOR },
	{ IDC_TEXT_STYLE, ResourceManager::SETCZDC_TEXT_STYLE },
	{ IDC_DEFAULT_STYLES, ResourceManager::SETCZDC_DEFAULT_STYLE },
	{ IDC_BLACK_AND_WHITE, ResourceManager::SETCZDC_BLACK_WHITE },
	{ IDC_BOLD_AUTHOR_MESS, ResourceManager::SETCZDC_BOLD },
	{ IDC_CZDC_PREVIEW, ResourceManager::SETCZDC_PREVIEW },
	{ IDC_SELWINCOLOR, ResourceManager::SETTINGS_SELECT_WINDOW_COLOR },
	{ IDC_SELTEXT, ResourceManager::SETTINGS_SELECT_TEXT_FACE },
	{ IDC_ERROR_COLOR, ResourceManager::SETCZDC_ERROR_COLOR },
	{ IDC_PAUSED_COLOR, ResourceManager::PAUSED_COLOR },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
}; 

PropPage::Item PropPageTextStyles::items[] = {
	{ IDC_BOLD_AUTHOR_MESS, SettingsManager::BOLD_AUTHOR_MESS, PropPage::T_BOOL },
	{ 0, 0, PropPage::T_END }
};

LRESULT PropPageTextStyles::onInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	PropPage::translate((HWND)(*this), texts);
	PropPage::read((HWND)*this, items);

	m_lsbList.Attach( GetDlgItem(IDC_TEXT_STYLES) );
	m_Preview.Attach( GetDlgItem(IDC_PREVIEW) );

	WinUtil::decodeFont(Text::toT(SETTING(TEXT_FONT)), m_Font);
	m_BackColor = SETTING(BACKGROUND_COLOR);
	m_ForeColor = SETTING(TEXT_COLOR);

	fg = SETTING(TEXT_COLOR);
	bg = SETTING(BACKGROUND_COLOR);
	err = SETTING(ERROR_COLOR);
	paused = SETTING(PAUSED_COLOR);
	TextStyles[ TS_GENERAL ].Init( 
	this, settings, "General text", "ABCD EFGH | General chat text",
	SettingsManager::TEXT_GENERAL_BACK_COLOR, SettingsManager::TEXT_GENERAL_FORE_COLOR, 
	SettingsManager::TEXT_GENERAL_BOLD, SettingsManager::TEXT_GENERAL_ITALIC );

	TextStyles[ TS_MYNICK ].Init( 
	this, settings, "My nick", "It's me | My nick",
	SettingsManager::TEXT_MYNICK_BACK_COLOR, SettingsManager::TEXT_MYNICK_FORE_COLOR, 
	SettingsManager::TEXT_MYNICK_BOLD, SettingsManager::TEXT_MYNICK_ITALIC );

	TextStyles[ TS_MYMSG ].Init( 
	this, settings, "My own message", "Hello world ! | My own message",
	SettingsManager::TEXT_MYOWN_BACK_COLOR, SettingsManager::TEXT_MYOWN_FORE_COLOR, 
	SettingsManager::TEXT_MYOWN_BOLD, SettingsManager::TEXT_MYOWN_ITALIC );

	TextStyles[ TS_PRIVATE ].Init( 
	this, settings, "Private message", "Could you give me a slot ? | Private message",
	SettingsManager::TEXT_PRIVATE_BACK_COLOR, SettingsManager::TEXT_PRIVATE_FORE_COLOR, 
	SettingsManager::TEXT_PRIVATE_BOLD, SettingsManager::TEXT_PRIVATE_ITALIC );

	TextStyles[ TS_SYSTEM ].Init( 
	this, settings, "System message", "File list refreshed | System message ",
	SettingsManager::TEXT_SYSTEM_BACK_COLOR, SettingsManager::TEXT_SYSTEM_FORE_COLOR, 
	SettingsManager::TEXT_SYSTEM_BOLD, SettingsManager::TEXT_SYSTEM_ITALIC);

	TextStyles[ TS_SERVER ].Init( 
	this, settings, "Server message", "*** Connecting to hub XYZ ...| Server message",
	SettingsManager::TEXT_SERVER_BACK_COLOR, SettingsManager::TEXT_SERVER_FORE_COLOR, 
	SettingsManager::TEXT_SERVER_BOLD, SettingsManager::TEXT_SERVER_ITALIC );

	TextStyles[ TS_TIMESTAMP ].Init( 
	this, settings, "Timestamp", "01.02.2345 12:34 | The style for timestamp",
	SettingsManager::TEXT_TIMESTAMP_BACK_COLOR, SettingsManager::TEXT_TIMESTAMP_FORE_COLOR, 
	SettingsManager::TEXT_TIMESTAMP_BOLD, SettingsManager::TEXT_TIMESTAMP_ITALIC );

	TextStyles[ TS_URL ].Init( 
	this, settings, "URL (http, ftp, ...)", "http://www.czdc.org | URL",
	SettingsManager::TEXT_URL_BACK_COLOR, SettingsManager::TEXT_URL_FORE_COLOR, 
	SettingsManager::TEXT_URL_BOLD, SettingsManager::TEXT_URL_ITALIC );

	TextStyles[ TS_FAVORITE ].Init( 
	this, settings, "Favorite user", "Popeye the Sailor Man | Favorite user",
	SettingsManager::TEXT_FAV_BACK_COLOR, SettingsManager::TEXT_FAV_FORE_COLOR, 
	SettingsManager::TEXT_FAV_BOLD, SettingsManager::TEXT_FAV_ITALIC );

	TextStyles[ TS_OP ].Init( 
	this, settings, "OP", "For ones GOD for others EVIL | OP",
	SettingsManager::TEXT_OP_BACK_COLOR, SettingsManager::TEXT_OP_FORE_COLOR, 
	SettingsManager::TEXT_OP_BOLD, SettingsManager::TEXT_OP_ITALIC );

	for ( int i = 0; i < TS_LAST; i++ ) {
		TextStyles[ i ].LoadSettings();
		_tcscpy(TextStyles[i].szFaceName, m_Font.lfFaceName );
		TextStyles[ i ].bCharSet = m_Font.lfCharSet;
		TextStyles[ i ].yHeight = m_Font.lfHeight;
		m_lsbList.AddString(Text::toT(TextStyles[i].m_sText).c_str() );
	}
	m_lsbList.SetCurSel( 0 );

	RefreshPreview();
	return TRUE;
}

void PropPageTextStyles::write()
{
	PropPage::write((HWND)*this, items);

	tstring f = WinUtil::encodeFont(m_Font);
	settings->set(SettingsManager::TEXT_FONT, Text::fromT(f));

	m_BackColor = TextStyles[ TS_GENERAL ].crBackColor;
	m_ForeColor = TextStyles[ TS_GENERAL ].crTextColor;

	settings->set(SettingsManager::TEXT_COLOR, (int)fg);
	settings->set(SettingsManager::BACKGROUND_COLOR, (int)bg);
	settings->set(SettingsManager::ERROR_COLOR, (int)err);
	settings->set(SettingsManager::PAUSED_COLOR, (int)paused);

	for ( int i = 0; i < TS_LAST; i++ ) {
		TextStyles[ i ].SaveSettings();
	}
	WinUtil::initColors();
}

LRESULT PropPageTextStyles::onEditBackColor(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	int iNdx = m_lsbList.GetCurSel();
	TextStyles[ iNdx ].EditBackColor();
	RefreshPreview();
	return TRUE;
}

LRESULT PropPageTextStyles::onEditForeColor(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	int iNdx = m_lsbList.GetCurSel();
	TextStyles[ iNdx ].EditForeColor();
	RefreshPreview();
	return TRUE;
}

LRESULT PropPageTextStyles::onEditTextStyle(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	int iNdx = m_lsbList.GetCurSel();
	TextStyles[ iNdx ].EditTextStyle();

	_tcscpy( m_Font.lfFaceName, TextStyles[ iNdx ].szFaceName );
	m_Font.lfCharSet = TextStyles[ iNdx ].bCharSet;
	m_Font.lfHeight = TextStyles[ iNdx ].yHeight;

	if ( iNdx == TS_GENERAL ) {
		if ( ( TextStyles[ iNdx ].dwEffects & CFE_ITALIC ) == CFE_ITALIC )
			m_Font.lfItalic = true;
		if ( ( TextStyles[ iNdx ].dwEffects & CFE_BOLD ) == CFE_BOLD )
			m_Font.lfWeight = FW_BOLD;
	}

	for ( int i = 0; i < TS_LAST; i++ ) {
		_tcscpy( TextStyles[ iNdx ].szFaceName, m_Font.lfFaceName );
		TextStyles[ i ].bCharSet = m_Font.lfCharSet;
		TextStyles[ i ].yHeight = m_Font.lfHeight;
		m_Preview.AppendText(_T("My nick"), _T("12:34 "), Text::toT(TextStyles[i].m_sPreviewText).c_str(), 
			TextStyles[i]);
	}

	RefreshPreview();
	return TRUE;
}

void PropPageTextStyles::RefreshPreview() {
	m_Preview.SetBackgroundColor( bg );
	m_Preview.SetTextStyleMyNick( TextStyles[ TS_MYNICK ] );
	m_Preview.SetWindowText(_T(""));

	string sText;
	for ( int i = 0; i < TS_LAST; i++ ) {
		m_Preview.AppendText(_T("My nick"), _T("12:34 "), Text::toT(TextStyles[i].m_sPreviewText).c_str(), 
		TextStyles[i] );
	}
	m_Preview.InvalidateRect( NULL );
}

LRESULT PropPageTextStyles::onDefaultStyles(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
	bg = RGB(0,0,96);
	fg = RGB(255,255,255);
	TextStyles[ TS_GENERAL ].crBackColor = RGB(0,0,96);
	TextStyles[ TS_GENERAL ].crTextColor = RGB(255,255,255);
	TextStyles[ TS_GENERAL ].dwEffects = 0;

	TextStyles[ TS_MYNICK ].crBackColor = RGB(0,0,96);
	TextStyles[ TS_MYNICK ].crTextColor = RGB(255,255,0);
	TextStyles[ TS_MYNICK ].dwEffects = CFE_BOLD;

	TextStyles[ TS_MYMSG ].crBackColor = RGB(96,0,0);
	TextStyles[ TS_MYMSG ].crTextColor = RGB(255,255,0);
	TextStyles[ TS_MYMSG ].dwEffects = CFE_BOLD;

	TextStyles[ TS_PRIVATE ].crBackColor = RGB(0,96,0);
	TextStyles[ TS_PRIVATE ].crTextColor = RGB(255,255,255);
	TextStyles[ TS_PRIVATE ].dwEffects = CFE_BOLD;

	TextStyles[ TS_SYSTEM ].crBackColor = RGB(0,0,0);
	TextStyles[ TS_SYSTEM ].crTextColor = RGB(192,192,192);
	TextStyles[ TS_SYSTEM ].dwEffects = CFE_BOLD | CFE_ITALIC;

	TextStyles[ TS_SERVER ].crBackColor = RGB(0,0,0);
	TextStyles[ TS_SERVER ].crTextColor = RGB(128,255,128);
	TextStyles[ TS_SERVER ].dwEffects = CFE_BOLD;

	TextStyles[ TS_TIMESTAMP ].crBackColor = RGB(0,0,0);
	TextStyles[ TS_TIMESTAMP ].crTextColor = RGB(255,255,0);
	TextStyles[ TS_TIMESTAMP ].dwEffects = 0;

	TextStyles[ TS_URL ].crBackColor = RGB(192,192,192);
	TextStyles[ TS_URL ].crTextColor = RGB(0,0,255);
	TextStyles[ TS_URL ].dwEffects = 0;

	TextStyles[ TS_FAVORITE ].crBackColor = RGB(0,0,0);
	TextStyles[ TS_FAVORITE ].crTextColor = RGB(255,128,128);
	TextStyles[ TS_FAVORITE ].dwEffects = CFE_BOLD | CFE_ITALIC;

	TextStyles[ TS_OP ].crBackColor = RGB(0,0,0);
	TextStyles[ TS_OP ].crTextColor = RGB(255,255,255);
	TextStyles[ TS_OP ].dwEffects = CFE_BOLD;

	RefreshPreview();
	return TRUE;
}

LRESULT PropPageTextStyles::onBlackAndWhite(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
	bg = RGB(255,255,255);
	fg = RGB(0,0,0);
	TextStyles[ TS_GENERAL ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_GENERAL ].crTextColor = RGB(0,0,96);
	TextStyles[ TS_GENERAL ].dwEffects = 0;

	TextStyles[ TS_MYNICK ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_MYNICK ].crTextColor = RGB(0,0,96);
	TextStyles[ TS_MYNICK ].dwEffects = 0;

	TextStyles[ TS_MYMSG ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_MYMSG ].crTextColor = RGB(0,0,96);
	TextStyles[ TS_MYMSG ].dwEffects = 0;

	TextStyles[ TS_PRIVATE ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_PRIVATE ].crTextColor = RGB(0,0,96);
	TextStyles[ TS_PRIVATE ].dwEffects = 0;

	TextStyles[ TS_SYSTEM ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_SYSTEM ].crTextColor = RGB(0,0,96);
	TextStyles[ TS_SYSTEM ].dwEffects = 0;

	TextStyles[ TS_SERVER ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_SERVER ].crTextColor = RGB(0,0,96);
	TextStyles[ TS_SERVER ].dwEffects = 0;

	TextStyles[ TS_TIMESTAMP ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_TIMESTAMP ].crTextColor = RGB(0,0,96);
	TextStyles[ TS_TIMESTAMP ].dwEffects = 0;

	TextStyles[ TS_URL ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_URL ].crTextColor = RGB(0,0,96);
	TextStyles[ TS_URL ].dwEffects = 0;

	TextStyles[ TS_FAVORITE ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_FAVORITE ].crTextColor = RGB(0,0,96);
	TextStyles[ TS_FAVORITE ].dwEffects = 0;

	TextStyles[ TS_OP ].crBackColor = RGB(255,255,255);
	TextStyles[ TS_OP ].crTextColor = RGB(0,0,0);
	TextStyles[ TS_OP ].dwEffects = CFE_BOLD;

	RefreshPreview();
	return TRUE;
}

void PropPageTextStyles::TextStyleSettings::Init( 
	PropPageTextStyles *pParent, SettingsManager *pSM, 
	LPCSTR sText, LPCSTR sPreviewText,
	SettingsManager::IntSetting iBack, SettingsManager::IntSetting iFore, 
	SettingsManager::IntSetting iBold, SettingsManager::IntSetting iItalic) {

	CHARFORMAT2 cf;
	cbSize = sizeof( cf );
	dwMask = CFM_COLOR | CFM_BOLD | CFM_ITALIC | CFM_BACKCOLOR;
	dwReserved = 0;
  
	m_pParent = pParent;
	settings = pSM;      
	m_sText = sText;
	m_sPreviewText = sPreviewText;
	m_iBackColor = iBack;
	m_iForeColor = iFore;
	m_iBold = iBold;
	m_iItalic = iItalic;
}

void PropPageTextStyles::TextStyleSettings::LoadSettings() {
	dwEffects = 0;
	crBackColor = settings->get( m_iBackColor );
	crTextColor = settings->get( m_iForeColor );
	if ( settings->get( m_iBold ) ) dwEffects |= CFE_BOLD;
	if ( settings->get( m_iItalic) ) dwEffects |= CFE_ITALIC;
}

void PropPageTextStyles::TextStyleSettings::SaveSettings() {
	settings->set( m_iBackColor, (int) crBackColor);
	settings->set( m_iForeColor, (int) crTextColor);
	BOOL boBold = ( ( dwEffects & CFE_BOLD ) == CFE_BOLD );
	settings->set( m_iBold, (int) boBold);
	BOOL boItalic = ( ( dwEffects & CFE_ITALIC ) == CFE_ITALIC );
	settings->set( m_iItalic, (int) boItalic);
}

void PropPageTextStyles::TextStyleSettings::EditBackColor() {
	CColorDialog d( crBackColor, 0, *m_pParent );
	if (d.DoModal() == IDOK) {
		crBackColor = d.GetColor();
	}
}

void PropPageTextStyles::TextStyleSettings::EditForeColor() {
	CColorDialog d( crTextColor, 0, *m_pParent );
	if (d.DoModal() == IDOK) {
		crTextColor = d.GetColor();
	}
}

void PropPageTextStyles::TextStyleSettings::EditTextStyle() {
	LOGFONT font;
	WinUtil::decodeFont(Text::toT(SETTING(TEXT_FONT)), font );

	_tcscpy( font.lfFaceName, szFaceName );
	font.lfCharSet = bCharSet;
	font.lfHeight = yHeight;

	if ( dwEffects & CFE_BOLD ) 
		font.lfWeight = FW_BOLD;
	else
		font.lfWeight = FW_REGULAR;

	if ( dwEffects & CFE_ITALIC ) 
		font.lfItalic = true;
	else
		font.lfItalic = false;

	CFontDialog d( &font, CF_SCREENFONTS, NULL, *m_pParent );
	d.m_cf.rgbColors = crTextColor;
	if (d.DoModal() == IDOK) {
  	_tcscpy( szFaceName, font.lfFaceName );
	bCharSet = font.lfCharSet;
	yHeight = font.lfHeight;

	crTextColor = d.m_cf.rgbColors;
	if ( font.lfWeight == FW_BOLD )
		dwEffects |= CFE_BOLD;
	else
		dwEffects &= ~CFE_BOLD;

	if ( font.lfItalic )
		dwEffects |= CFE_ITALIC;
	else
		dwEffects &= ~CFE_ITALIC;
	}
}

LRESULT PropPageTextStyles::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
	if (m_lsbList.m_hWnd != NULL)
		m_lsbList.Detach();
	if (m_Preview.m_hWnd != NULL)
		m_Preview.Detach();
	return 1;
}

LRESULT PropPageTextStyles::onEditBackground(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
	CColorDialog d( bg, 0, *this );
	if (d.DoModal() == IDOK) {
		bg = d.GetColor();
	}
	RefreshPreview();
	return TRUE;
}

LRESULT PropPageTextStyles::onClickedText(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	LOGFONT tmp = m_Font;
	CFontDialog d(&tmp, CF_EFFECTS | CF_SCREENFONTS, NULL, *this);
	d.m_cf.rgbColors = fg;
	if(d.DoModal() == IDOK)
	{
		m_Font = tmp;
		fg = d.GetColor();
	}
	RefreshPreview();
	return TRUE;
}

LRESULT PropPageTextStyles::onEditError(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
	CColorDialog d( err, 0, *this );
	if (d.DoModal() == IDOK) {
		err = d.GetColor();
	}
	return TRUE;
}

LRESULT PropPageTextStyles::onEditPaused(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
	CColorDialog d(paused, 0, *this);
	if (d.DoModal() == IDOK) {
		paused = d.GetColor();
	}
	return TRUE;
}
